from dnxdata.resource import lambda_client
from dnxdata.logger import Logger


class Lambda:

    def __init__(self):
        self.logger = Logger("DNX Lambda => ")

    def invoke(self, name):
        self.logger.info("Starting Invoke Lambda")
        response = lambda_client.invoke_lambda.invoke(
            FunctionName=name,
            InvocationType='Event'
        )

        if response.get("StatusCode") == 202:
            self.logger.info("Invoke Lambda Success")
        else:
            self.logger.info(
                "Failed Invoke Lambda {}"
                .format(response)
            )

        self.logger.info("Finishing Invoke Lambda")



import dateutil.tz
from datetime import datetime
from dnxdata.logger import Logger


class Utils:

    def __init__(self):
        self.logger = Logger("DNX Utils => ")

    def get_bucket_key(self, path):
        """
        Ex:
        path: 's3://bucket/folder0/folder1/key.csv'
        return
        bucket: 'bucket'
        key: 'folder0/folder1'
        """

        path = path.strip().replace("s3://", "")
        bucket = path.strip().split("/")[0]
        key = "/".join(path.split("/")[1:-1])

        source = {"bucket": bucket.strip(), "key": key.strip()}
        self.logger.debug(
            "Get Bucket key source {}"
            .format(source)
        )
        return source

    def get_path_file_processed(self, path, file_status):
        """
        Ex:
        path: 's3://bucket/folder0/folder1/key.csv'
        status: SUCCEEDED
        return
        key_dest: 'SUCCEED_20200522_164047_file.gz'
        """
        path = path.replace("s3://", "")
        key = "".join((("/".join(path.split("/")[1:])).split("/")[-1:]))

        time_format = "%Y%m%d_%H%M%S"
        key_dest = "{}_{}_.".format(
                        file_status,
                        self.date_time(format_date=False).strftime(time_format)
                    )
        key_dest = key_dest.replace(".", key)

        return key_dest

    def date_time(self, format_date=True,
                  timezone="Australia/Sydney", milliseconds=False):
        eastern = dateutil.tz.gettz(timezone)
        dt = datetime.now(tz=eastern)

        if format_date:
            if milliseconds:
                dt = dt.strftime("%Y-%m-%d %H:%M:%S:%f")
            else:
                dt = dt.strftime("%Y-%m-%d %H:%M:%S")
        return dt

    def separate_path(self, path):
        """
        Ex:
        path: 's3://bucket/folder0/folder1/key.csv'
        return
        bucket: 'bucket'
        key: 'folder0/folder1/key.csv'
        """
        path = path.strip().replace("s3://", "")
        bucket = path.strip().split("/")[0]
        key = "/".join(path.split("/")[1:])

        return bucket, key



import gzip
import time
from dnxdata.utils.utils import Utils
from dnxdata.logger import Logger
from dnxdata.resource import s3_client, s3_resource
from io import BytesIO


class Boto3:

    def __init__(self):
        self.utils = Utils()
        self.logger = Logger("DNX Boto3 => ")

    def get_list_parquet(self, path):
        self.logger.info("Starting get_list_parquet")

        keys_s3 = []
        list_path = []

        v_boolean_list = True if isinstance(path, list) else False

        while True:
            if not v_boolean_list:
                if path.endswith('.parquet'):
                    keys_s3.append(path)
                    break
                else:
                    list_path.append(path)
            else:
                list_path = path

            for row in list_path:
                if row.endswith('.parquet'):
                    keys_s3.append(row)
                    continue
                source = self.utils.get_bucket_key(row)
                parquets = self.get_list_file(
                    bucket=source.get("bucket"),
                    filepath=source.get("key"),
                    endswith=".parquet"
                )
                for parquet in parquets:
                    keys_s3.append(parquet)
            break
        self.logger.info("Finishing get_list_parquet")
        return keys_s3

    def put_object_s3(self, bucket, key, file):

        self.logger.info("Starting put_object_s3")

        gz_body = BytesIO()
        gz = gzip.GzipFile(None, 'wb', 9, gz_body)
        gz.write(str(file).encode('utf-8'))  # convert unicode strings to bytes
        gz.close()
        # GzipFile has written the compressed bytes into our gz_body
        s3_client.put_object(
            Bucket=bucket,
            Key=key,  # Note: NO .gz extension!
            ContentType='text/plain',  # the original type
            ContentEncoding='gzip',  # MUST have or browsers will error
            Body=gz_body.getvalue()
        )

        self.logger.info("Finishing put_object_s3")

    def get_object_s3(self, bucket, key, format_file=None):

        self.logger.info(
            "Starting GetObjectS3 format {} {}/{}"
            .format(
                format_file,
                bucket,
                key
            )
        )

        file = s3_client.get_object(Bucket=bucket, Key=key)

        if format_file is not None:
            if format_file.lower() == "gz":
                bytestream = BytesIO(file['Body'].read())
                decoded = gzip.GzipFile(
                    None,
                    'rb',
                    fileobj=bytestream
                ).read().decode('utf-8')
        else:
            file = file['Body'].read()
            decoded = file.decode('utf-8')

        self.logger.info("Finishing GetObjectS3")

        return decoded

    def move_file_s3(self, bucket_ori, key_ori, bucket_dest, key_dest):
        self.logger.info(
            "Starting MoveFileS3 Ori {}/{}  Dest {}/{}"
            .format(
                bucket_ori,
                key_ori,
                bucket_dest,
                key_dest
            )
        )

        copy_source = {'Bucket': bucket_ori, 'Key': key_ori}
        self.logger.info("Starting Change Bucket")
        s3_client.copy_object(
            CopySource=copy_source,
            Bucket=bucket_dest.strip(),
            Key=key_dest.strip()
        )
        self.logger.info("Finishing Change Bucket")
        time.sleep(3)
        self.logger.info("Starting Delete File")
        s3_client.delete_object(
            Bucket=bucket_ori.strip(),
            Key=key_ori.strip()
        )
        self.logger.info("Finishing Delete File")

        self.logger.info("Finishing MoveFileS3")

    def _delete_key(self, bucket, key):

        v_count = 0
        while v_count < 3:
            v_count += 1
            try:
                s3_resource.Object(bucket, key).delete()
                time.sleep(1)
            except Exception as e:
                self.logger.error(e)
                pass

    def delete_file_s3(self, path, path_or_key="key"):

        self.logger.debug(
            "Starting DeleteFileS3 path {} path_or_key {}"
            .format(
                path,
                path_or_key
            )
        )

        while True:

            if not isinstance(path, list):
                path = path.split(" ")

            if len(path) == 0:
                self.logger.debug("0 path for delete")
                break

            bucket_aux = ""
            for row in path:
                _bucket, _key = self.utils.separate_path(path=row)
                if _bucket != bucket_aux:
                    bucket = s3_resource.Bucket(_bucket)
                    bucket_aux = _bucket

                if path_or_key.lower() == "path":
                    for obj in bucket.objects.filter(Prefix=_key):
                        self.logger.debug(
                            "File deleted {}/{}"
                            .format(
                                _bucket,
                                obj.key
                            )
                        )
                        self._delete_key(bucket=_bucket, key=obj.key)

                elif path_or_key.lower() == "key":
                    self.logger.debug(
                        "File deleted {}/{}"
                        .format(
                            _bucket,
                            _key
                        )
                    )
                    self._delete_key(bucket=_bucket, key=_key)
            break

        self.logger.debug("Finishing DeleteFileS3")

    def get_list_file(self, bucket, filepath, endswith):

        self.logger.debug(
            "Starting GetListFile {}/{}/*{}"
            .format(
                bucket,
                filepath,
                endswith
            )
        )

        if not filepath.endswith('/'):
            filepath += '/'

        """
        path = "{}/{}".format(bucket, filepath)
        desc = wr.s3.describe_objects(path, wait_time=30)
        S3keys = [key for key in desc if key.endswith(endswith)]
        This call above is problem.
        """
        prefix = filepath[1:] if filepath.startswith("/") else filepath
        _bucket = s3_resource.Bucket(bucket)

        keys_s3 = []
        for _ in _bucket.objects.filter(Prefix=prefix):
            if _.key.endswith(endswith):
                keys_s3.append(["s3://" + str(bucket) + "/" + _.key])

        if (len(keys_s3) == 0):
            self.logger.debug(
                "No {} found in bucket {}/{}"
                .format(
                    endswith,
                    bucket,
                    filepath
                )
            )
        else:
            for p in keys_s3:
                self.logger.debug("S3Keys {}".format(p))

        self.logger.debug("Finishing GetListFile")

        return keys_s3



import boto3
import uuid
from dnxdata.logger import Logger


class Kinesis:

    def __init__(self, name, region):
        self.name = name
        self.region = region
        self.logger = Logger("DNX Kinesis => ")

    def conn(self):

        self.logger.info(
            "Starting Conn Kinesis name {}, region {}"
            .format(
                self.name,
                self.region
            )
        )

        if self.region and self.name:

            self.client = boto3.client('firehose', region_name=self.region)
            self.stream = self.name.format(uuid.uuid4())

            self.kinesis = {"client": self.client,
                            "stream": self.stream}

            self.logger.info("Finishing Conn Kinesis")
            return self.kinesis
        else:
            self.logger.info("Invalid argument Conn Kinesis")
            exit(1)

    def send_data(self, data_binary_string, put_record="PRB"):

        self.logger.debug("Starting SendData Kinesis")

        option = ["PR", "PRB"]
        if put_record not in option:
            self.logger.error(
                "Put record not configure. Available {}"
                .format(
                    option
                )
            )
            exit(1)

        if put_record == "PR":

            data_string = str(data_binary_string)
            data_binary_string = str.encode(data_string)
            response = self.client.put_record(
                DeliveryStreamName=self.stream,
                Record={'Data': data_binary_string}
            )

        elif put_record == "PRB":

            data_string = str(data_binary_string) + "\n"
            data_binary_string = str.encode(data_string)
            response = self.client.put_record_batch(
                DeliveryStreamName=self.stream,
                Records=[{'Data': data_binary_string}]
            )

        if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
            self.logger.debug(
                "SUCCESS, your request ID is : {}"
                .format(
                    response["ResponseMetadata"]["RequestId"]
                )
            )

        else:
            self.logger.error("ERROR : something went wrong")
            exit(1)

        self.logger.debug("Finishing SendData Kinesis")



import boto3
from dnxdata.utils.utils import Utils
from dnxdata.logger import Logger
from dnxdata.resource import glue_client


class Glue:

    def __init__(self):
        self.utils = Utils()
        self.logger = Logger("DNX Glue => ")

    def get_job_glue(self, job_name, job_run_id):

        self.logger.info(
            "Starting _GetJobGlue job_name {},job_run_id {}"
            .format(
                job_name,
                job_run_id
            )
        )

        status = glue_client.get_job_run(JobName=job_name, RunId=job_run_id)

        self.logger.info(
            "Finishing _GetJobGlue status {}"
            .format(status)
        )

        return status

    def get_status_job_glue(self, job_name, job_run_id):

        self.logger.info(
            "Starting GetStatusJobGlue job_name {}, job_run_id {}"
            .format(
                job_name,
                job_run_id
            )
        )
        status = self.get_job_glue(job_name=job_name, job_run_id=job_run_id)
        state = status['JobRun']['JobRunState']

        self.logger.info(
            "Finishing GetStatusJobGlue Status {}"
            .format(state)
        )

        return state

    def get_msg_error_job_glue(self, job_name, job_run_id):

        self.logger.info(
            "Starting GetMsgErrorJobGlue job_name {}, job_run_id {}"
            .format(
                job_name,
                job_run_id
            )
        )
        status = self.get_job_glue(job_name=job_name, job_run_id=job_run_id)
        msg_error = status['JobRun']['ErrorMessage']

        self.logger.info(
            "Finishing GetMsgErrorJobGlue ErrorMessage {}"
            .format(msg_error)
        )

        return msg_error



from dnxdata.utils.utils import Utils
from dnxdata.utils.boto3 import Boto3
from dnxdata.logger import Logger
import awswrangler as wr
import pandas as pd


class Pandas:

    def __init__(self, region=None):
        self.region = region
        self.utils = Utils()
        self.s3 = Boto3()
        self.logger = Logger("DNX Pandas => ")

    # You can pass list or string path or .parquet
    def get_parquet(self, path):
        self.logger.info("Starting GetParquet {}".format(path))

        keys_s3 = self.s3.get_list_parquet(path)
        self.logger.info("S3keys {}".format(keys_s3))
        if len(keys_s3) == 0:
            self.logger.info("Finishing GetParquet")
            return pd.DataFrame()

        df = wr.s3.read_parquet(
            path=keys_s3,
            dataset=True,
            validate_schema=False
        )

        self.logger.info("Finishing GetParquet")

        return df

    def write_parquet(self, df, path, database, table, mode, partition_cols, list_path_delete):
        self.logger.info("Starting Write Parquet")

        if (list_path_delete is not None) & (mode != "append"):
            self.logger.info(
                "list_path_delete => {}"
                .format(list_path_delete)
            )
            self.s3.delete_file_s3(
                path=list_path_delete,
                path_or_key="key"
            )

        if df.empty:
            self.logger.info("DF Write Parquet None or only lines with delete")
        else:
            df["dt_process_parq"] = self.utils.date_time()
            df["dt_process_parq"] = pd.to_datetime(df["dt_process_parq"])

            if partition_cols:
                wr.s3.to_parquet(
                    df=df,
                    path=path,
                    dataset=True,
                    database=database,
                    table=table,
                    mode=mode,
                    partition_cols=partition_cols,
                    compression='snappy'
                )
            else:
                wr.s3.to_parquet(
                    df=df,
                    path=path,
                    dataset=True,
                    database=database,
                    table=table,
                    mode=mode,
                    compression='snappy'
                )

        self.logger.info("Finishing Write Parquet")

    def print_dtypes(self, df):
        self.logger.debug("Starting print_dtypes")

        result = {}
        try:
            result = {k: str(v) for k, v in df.dtypes.items()}
        except Exception as e:
            self.logger.warning(e)
            pass

        index = {}
        try:
            index = {row for row in df.index.names}
        except Exception as e:
            self.logger.warning(e)
            pass

        self.logger.debug("List dtypes")
        self.logger.debug(result)
        self.logger.debug("List index")
        self.logger.debug(index)

        self.logger.debug("Finishing GetDtypesDf")



import time
from boto3.dynamodb.conditions import Key
from dnxdata.utils.utils import Utils
from dnxdata.resource import dynamo_resource
from dnxdata.logger import Logger


class Dynamo:

    def __init__(self):
        self.utils = Utils()
        self.logger = Logger("DNX Dynamo => ")

    def put_table_item(self, table, item, key, update=True):

        self.logger.info(
            "Starting PutTableItem Table {} DynamoDB Item {}"
            .format(
                table,
                item
            )
        )

        # TODO review logic
        if update:
            item_get = self.get_item_table(table=table, key=key)
            if len(item_get) != 0:
                item_get.update({k: v for k, v in item.items()})
            else:
                item_get = item
        else:
            item_get = item

        item_get.update({"timestamp": self.utils.date_time(milliseconds=True)})
        table_db = dynamo_resource.Table(table)
        response = table_db.put_item(Item=item_get)

        debug(
            "Finishing PutTableItem Table {} DynamoDB Response {}"
            .format(
                table,
                response
            )
        )

        self.logger.info("Finishing PutTableItem")

        return response

    def get_item_table(self, table, key):

        self.logger.info(
            "Starting GetItemTable Table {} DynamoDB Key {}"
            .format(
                table,
                key
            )
        )
        table_db = dynamo_resource.Table(table)
        response = table_db.get_item(Key=key)
        # TODO Review logic
        result = {k: v for k, v in response.get("Item").items() if k not in ['ResponseMetadata']} if response.get("Item") else {}

        debug("{}".format(result))

        self.logger.info("Finishing GetItemTable")

        return result

    def delete_table_item(self, table, key):

        self.logger.info(
            "Starting DeleteTableItem Table {} DynamoDB Key {}"
            .format(
                table,
                key
            )
        )
        table_db = dynamo_resource.Table(table)
        response = table_db.delete_item(Key=key)

        if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
            self.logger.info("Successful deletion")
            time.sleep(1)
            item = self.get_item_table(table=table, key=key)
            if len(item) > 1:
                self.logger.info("Double check exclusion table")
                time.sleep(1)
                response = table_db.delete_item(Key=key)

        self.logger.info(
            "Finishing DeleteTableItem Table {} DynamoDB Response {}"
            .format(
                table,
                response
            )
        )

        return response

    def update_table_item(self, table, key, updateExpression, expressionAttributeValues):

        try:
            self.logger.info(
                "Starting UpdateTableItem Table {} DynamoDB, Key {}, updateExpression {}, expressionAttributeValues {}"
                .format(
                    table,
                    key,
                    updateExpression,
                    expressionAttributeValues
                )
            )
            table_db = dynamo_resource.Table(table)

            response = table_db.update_item(
                Key=key,
                UpdateExpression=str(expressionAttributeValues),
                ExpressionAttributeValues=updateExpression,
                ReturnValues="UPDATED_NEW"
            )
            self.logger.info(
                "Finishing UpdateTableItem Table {} DynamoDB Response {}"
                .format(
                    table,
                    response
                )
            )
        except Exception as e:
            error(
                "UpdateTableItem Table: {} Error: {}"
                .format(
                    table,
                    e
                )
            )
            return
        return response

    def get_data_dynamo_index(self, table, index_name, key, value):

        self.logger.info(
            "Starting GetDataIndex Table {} DynamoDB, IndexName {}, Key {}, Value {}"
            .format(
                table,
                index_name,
                key,
                value
            )
        )
        table = dynamo_resource.Table(table)
        response = table.query(
            IndexName=index_name,
            KeyConditionExpression=Key(key).eq(value))
        response = response['Items']

        self.logger.info(
            "Finishing GetDataIndex table {} DynamoDB Response {}"
            .format(
                table,
                response
            )
        )

        return response

    def move_data_for_another_table(self, key, list_update, table_ori, table_dest, delete_ori=True):

        self.logger.info(
            "Starting MoveDataForOtherTable Key {}, listUpdate {}, tableOri {},tableDest {}"
            .format(
                key,
                list_update,
                table_ori,
                table_dest
            )
        )
        item = self.get_item_table(table=table_ori, key=key)

        if len(item) != 0:
            self.logger.info("MoveDataForOtherTable Line Log StageControl")
            # TODO Review logic
            item.update({k: v for k, v in list_update.items()})
            response = self.put_table_item(
                table=table_dest,
                item=item,
                key=key,
                update=False
            )
            self.logger.info(
                "MoveDataForOtherTable Response PutTableItem {}"
                .format(
                    response
                )
            )

            if not response and delete_ori:
                time.sleep(2)
                response = self.delete_table_item(
                    table=table_ori,
                    key=key
                )
                self.logger.info(
                    "MoveDataForOtherTable Response DeleteTableItem {}"
                    .format(
                        response
                    )
                )
            else:
                self.logger.info(
                    "self.logger.infodelete_ori {} "
                    .format(delete_ori))
                self.logger.info(
                    "MoveDataForOtherTable Invalid Response"
                )
        else:
            self.logger.info("MoveDataForOtherTable Invalid GetItemTable")

        self.logger.info("Finishing MoveDataForOtherTable")

    def scan_table_all_pages(self, table, filter_key=None, filter_value=None):

        self.logger.info(
            "Starting ScanTableAllPages Table {},filter_key {},filter_value {}"
            .format(
                table,
                filter_key,
                filter_value
            )
        )

        table = dynamo_resource.Table(table)
        items = []

        if filter_key and filter_value:
            if not isinstance(filter_value, list):
                self.logger.info(
                    "ScanTableAllPages Parameter invalid {}, required list []"
                    .format(filter_value))
                filter_value = filter_value.split(",")

            for row in filter_value:
                filtering_exp = Key(filter_key).eq(row)
                response = table.scan(FilterExpression=filtering_exp)

                items += response['Items']
                while True:
                    if response.get('LastEvaluatedKey'):
                        response = table.scan(
                            ExclusiveStartKey=response['LastEvaluatedKey']
                        )
                        items += response['Items']
                    else:
                        break

        else:
            response = table.scan()
            if len(response) != 0:
                items += response['Items']
                while True:
                    if response.get('LastEvaluatedKey'):
                        response = table.scan(
                            ExclusiveStartKey=response['LastEvaluatedKey']
                        )
                        items += response['Items']
                    else:
                        break

        self.logger.info(
            "self.logger.infoCount ScanTableAllPages {}"
            .format(len(items))
        )
        debug(
            "self.logger.infoItems Return ScanTableAllPages {}"
            .format(items)
        )

        self.logger.info("self.logger.infoFinishing ScanTableAllPages")

        return items



import sys
from pyspark import SparkFiles
from pyspark.sql import SparkSession
from pyspark.context import SparkContext
from awsglue.context import GlueContext


class ModSparkSession:

    def __init__(self):
        pass

    def start(self):
        self.spark = SparkSession.builder \
                                 .appName("Metrics-Common - Data ETL Glue") \
                                 .enableHiveSupport() \
                                 .getOrCreate()
        sys.path.insert(0, SparkFiles.getRootDirectory())

        self.log4jLogger = self.spark._jvm.org.apache.log4j
        self.glue_context = GlueContext(SparkContext.getOrCreate())
        self.spark.conf.set(
            "spark.sql.legacy.allowCreatingManagedTableUsingNonemptyLocation",
            "true"
        )
        return [self.spark, self.log4jLogger, self.glue_context]



import boto3

# S3
s3_client = boto3.client('s3')
s3_resource = boto3.resource('s3')

# Dynamodb
dynamo_client = boto3.client('dynamodb')
dynamo_resource = boto3.resource('dynamodb')

# Glue
glue_client = boto3.client("glue")

# Lambda
lambda_client = boto3.client('lambda')

# SSM
ssm_client = boto3.client('ssm')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


install_requires = [
    'dnx-mysql-replication==0.21'
    # 'awswrangler-git @ https://github.com/awslabs/aws-data-wrangler/releases/download/1.5.0/awswrangler-layer-1.5.0-py3.8.zip#egg=awswrangler-git-1.5.0'
]

setuptools.setup(
    name='dnx-data',  # Replace with your own username
    version='0.1.6',
    author='DNX Solutions',
    author_email='contact@dnx.solutions',
    description='DNX data solution package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/DNXLabs/dnx-data',
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)



import logging
import os


class Logger:

    def __init__(self, header):
        self.header = header
        logging.basicConfig()
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(os.getenv('LOG_LEVEL', 'INFO'))

    def info(self, msg, *args, **kwargs):
        self.logger.info(self.header + msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        self.logger.debug(self.header + msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        self.logger.error(self.header + msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        self.logger.error(self.header + msg, *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        self.logger.error(self.header + msg, *args, **kwargs)



import sys
from pyspark.sql.functions import lit
from pyspark.sql.functions import (when, col, trim)
from pyspark.sql import functions as F
from dnxdata.utils.utils import Utils
from dnxdata.utils.boto3 import Boto3


class SparkUtils:

    def __init__(self, spark, glue_context, log4jLogger):
        self.spark = spark
        self.log4jLogger = log4jLogger
        self.glue_context = glue_context
        self.utils = Utils()
        self.s3 = Boto3()

    def save_table(self, df, path, partition_column, mode, database, table, list_path_delete):
        # TODO Why using another log manager?
        logger = self.log4jLogger.LogManager.getLogger(__name__ + "." + self.__class__.__name__ + "." + sys._getframe().f_code.co_name)

        logger.info("Metrics-Common => Starting save table")
        logger.info(
            "Metrics-Common => Database + Table {}.{}"
            .format(
                database,
                table
            )
        )

        self.is_empty_df(df=df, exit_none=True)

        self.spark.catalog.setCurrentDatabase(database)

        df = df.withColumn("dt_process_parq", lit(self.utils.date_time()).cast("timestamp"))

        if len(list_path_delete) > 0:
            self.s3.delete_file_s3(path=list_path_delete, path_or_key="path")

        write_obj = df.write. \
            format("parquet"). \
            option("path", path). \
            option("mergeSchema", "true"). \
            option("compression", "snappy"). \
            mode(mode.replace("merge", "append"))

        if len(partition_column) > 0:
            write_obj = write_obj.partitionBy(partition_column)

        write_obj.saveAsTable(table)

        self.spark.catalog.refreshTable("{}.{}".format(database, table))

        logger.info("Metrics-Common => Finishing save table")

    def is_empty_df(self, df, exit_none=False):
        # TODO Why using another log manager?
        logger = self.log4jLogger.LogManager.getLogger(
            __name__ + "." + self.__class__.__name__ + "." + sys._getframe().f_code.co_name
        )

        logger.info("Metrics-Common => Starting IsEmptyDf")

        v_invalid_df = True if df.rdd.isEmpty() else False

        if exit_none & v_invalid_df:
            logger.info("Metrics-Common => IsEmptyDf Invalid Data Frame v_invalid_df {}".format(v_invalid_df))
            sys.exit(1)
            return

        logger.info(
            "Metrics-Common => Finishing IsEmptyDf v_invalid_df {}"
            .format(v_invalid_df)
        )

        return v_invalid_df

    def union_data_frame(self, df1, df2):
        # TODO Why using another log manager?
        logger = self.log4jLogger.LogManager.getLogger(__name__ + "." + self.__class__.__name__ + "." + sys._getframe().f_code.co_name)

        logger.info("Metrics-Common => Starting UnionDataFrame")
        logger.info("Metrics-Common => count 1 {}".format(df1.count()))
        logger.info("Metrics-Common => count 2 {}".format(df2.count()))

        # Add missing columns to df1
        df_left = df1
        for column in set(df2.columns) - set(df1.columns):
            df_left = df_left.withColumn(column, F.lit(None))

        # Add missing columns to df2
        df_right = df2
        for column in set(df1.columns) - set(df2.columns):
            df_right = df_right.withColumn(column, F.lit(None))

        logger.info("Metrics-Common => Finishing UnionDataFrame")

        # Make sure columns are ordered the same
        df = df_left.union(df_right.select(df_left.columns))
        logger.info("Metrics-Common => count 3 {}".format(df.count()))
        return df

    def fix_dtypes(self, df, list_col_dtypes, col_name_last_update):
        logger = self.log4jLogger.LogManager.getLogger(__name__ + "." + self.__class__.__name__ + "." + sys._getframe().f_code.co_name)

        logger.info("Metrics-Common => Starting FixValueNull")
        logger.info("Metrics-Common => list_col_dtypes {}".format(list_col_dtypes))

        dtypes = df.dtypes
        for row in dtypes:
            column = row[0]
            _dtypes = list_col_dtypes.get(column)

            if _dtypes is None:
                logger.info("Metrics-Common => Column not parameterized {}".format(row))
                continue

            if (_dtypes in ["double", "integer", "float"]) | (_dtypes.find("decimal") >= 0):
                logger.info("Metrics-Common => FixValueNull Number row {}".format(row))
                df = df.withColumn(column, when(trim(col(column)).isNull(), "0")
                                   .otherwise(trim(col(column))))
                df = df.withColumn(column, col(column).cast(_dtypes))
            elif _dtypes == "string":
                logger.info("Metrics-Common => FixValueNull String row {}".format(row))
                df = df.withColumn(column, trim(col(column)))

            elif _dtypes == "timestamp":
                logger.info("Metrics-Common => FixValueNull Timestamp row {}".format(row))
                if column == col_name_last_update:
                    df = df.withColumn(column, trim(col(column)).cast(_dtypes))
                else:
                    df = df.withColumn(column, trim(col(column)).cast("string"))

        logger.info("Metrics-Common => Finishing FixValueNull")
        return df

    def get_data_rds(self, connection_type, connection_settings):

        logger = self.log4jLogger.LogManager.getLogger(__name__ + "." + self.__class__.__name__ + "." + sys._getframe().f_code.co_name)
        logger.info("Metrics-Common => Starting get_data_rds")

        if connection_type == "mysql":
            conn = {
                "url": "jdbc:mysql://{}:{}/{}".format(
                    connection_settings.get("host"),
                    connection_settings.get("port"),
                    connection_settings.get("schema")
                ),
                "dbtable": connection_settings.get("dbtable"),
                "user": connection_settings.get("user"),
                "password": connection_settings.get("passwd")}

            df = self.glue_context.create_dynamic_frame.from_options(
                connection_type=connection_type,
                connection_options=conn
            )

            df = df.toDF()

        logger.info("Metrics-Common => Start Show Data frame RDS")
        df.printSchema()
        df.show()
        logger.info("Metrics-Common => Finish Show Data frame RDS")

        logger.info("Metrics-Common => Finishing get_data_rds")

        return df



import json
from dnxdata.logger import Logger
from dnxdata.utils.boto3 import Boto3


class Json:

    def __init__(self, bucket_stage_artifacts):
        self.bucket_stage_artifacts = bucket_stage_artifacts
        self.logger = Logger("DNX Json => ")

    def load_json(self, key=None, value=None):

        try:
            self.logger.info(
                "Starting LoadJson File Json Key {} Value {}"
                .format(
                    key,
                    value
                )
            )
            bucket = self.bucket_stage_artifacts
            key_s3 = "json/param_lambda.json"

            s3 = Boto3()
            file_json = s3.get_object_s3(bucket, key_s3)
            config_dict = json.loads(file_json)
            dict_load = config_dict

            if key is None:
                pass
            elif (key is not None) & (value is None):
                dict_load = dict_load[key]
            elif value is not None:
                dict_load = dict_load[key][value]

            self.logger.info("Finishing LoadJson File Json")
            self.logger.debug(print(dict_load))
            return dict_load

        except Exception as e:
            self.logger.error(
                "Error Load File Json: {} "
                .format(e)
            )

    def valid_key(self, key, value=None, table=None):

        self.logger.info(
            "Starting ValidKey Key {} Value {} Table {}"
            .format(
                key,
                value,
                table
            )
        )

        dict_file = self.load_json(key=key, value=value)

        equal_upper = table.upper() in dict_file.keys()
        equal_lower = table.lower() in dict_file.keys()

        if equal_upper or equal_lower:
            v_boolean = True
        else:
            v_boolean = False

        if not v_boolean:
            self.logger.info("Non-parameterized File the Json")

        self.logger.info("Finishing ValidKey Key File Json")

        return v_boolean

    def get_config(self, database, table):

        table = table.upper()
        database = database.lower()

        self.logger.debug(
            "Starting get_config Json Database {} Table {}".format(
                database,
                table
            )
        )

        config_default = {}

        file_json = self.load_json()

        try:
            config_default = file_json["param"]
            self.logger.debug("Config Param {}".format(config_default))
        except Exception as e:
            self.logger.error("Config Param not configured")
            self.logger.error(e)
            exit(1)

        try:
            config_default_aux = file_json["database"][database]["config"]
            config_default.update(config_default_aux)
            self.logger.debug(
                "Config Default Database {}"
                .format(
                    config_default_aux
                )
            )
        except Exception as e:
            self.logger.error(
                "Config Default not configured for the database {} table {}"
                .format(
                    database,
                    table
                )
            )
            self.logger.error(e)
            exit(1)

        try:
            config_default_aux = file_json["database"][database]["table"][table]
            config_default.update(config_default_aux)
            config_default.update({"database_rds": database})

            self.logger.debug(
                "Config Table {}"
                .format(config_default_aux)
            )
        except Exception as e:
            self.logger.debug(
                "Error Return Config Default {}"
                .format(str(e))
            )
            exit(1)

        self.logger.debug(
            "Return config, default and exception {}"
            .format(config_default)
        )

        return config_default




CANCELED = "CANCELED"
DECOMPRESSING = "DECOMPRESSING"
DELETED = "DELETED"
FAILED = "FAILED"
FREE = "FREE"
INITIAL_LOAD = "INITIAL_LOAD"
INVALID_FILE = "INVALID_FILE"
LOST = "LOST"
STAGE = "STAGE"
PROCESSING = "PROCESSING"
RUNNING = "RUNNING"
SUCCEEDED = "SUCCEEDED"
STOPPED = "STOPPED"
TIMEOUT = "TIMEOUT"
WAITING = "WAITING"



import pymysql
from dnxdata.logger import Logger


class Mysql:

    def __init__(self):
        self.logger = Logger("DNX Mysql => ")

    def execute(self, connection_settings, script):

        self.logger.debug(
            "Starting Exec Query Mysql Script {}"
            .format(script)
        )

        conn = self.connection(connection_settings=connection_settings)

        result = []
        with conn.cursor() as cur:
            cur.execute(script)
            conn.commit()
            cur.close()
            for row in cur:
                result.append(list(row))

        self.logger.debug("Return Mysql {}".format(result))
        self.logger.debug("Finishing Exec Query Mysql")
        return result

    def connection(self, connection_settings):

        self.logger.debug("Starting Connection Mysql")

        conn = pymysql.connect(
            host=connection_settings.get("host"),
            user=connection_settings.get("user"),
            passwd=connection_settings.get("passwd"),
            db=connection_settings.get("db"),
            connect_timeout=5
        )

        self.logger.debug("Finishing Connection Mysql")
        return conn

    def get_primary_key(self, connection_settings, schema, table):

        self.logger.debug(
            "Starting get_primary_key schema {} table {}"
            .format(
                schema,
                table
            )
        )

        select = """
        SELECT column_name
          FROM self.logger.information_schema.columns
          WHERE lower(table_schema) = "{}"
            AND lower(table_name) = "{}"
            AND COLUMN_KEY = "PRI"
          ORDER BY ORDINAL_POSITION """.format(schema.lower(), table.lower())

        self.logger.debug("Select get_primary_key {}".format(select))

        result = self.execute(
            connection_settings=connection_settings,
            script=select
        )

        # TODO Review logic
        result = [x for x in result[0]]

        self.logger.debug("Result {}".format(result))
        self.logger.debug("Finishing get_primary_key")

        return result
